////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007-2026 Laurent Gomila (laurent@sfml-dev.org)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <CSFML/Graphics/ConvertRect.hpp>
#include <CSFML/Graphics/Rect.h>
#include <CSFML/System/ConvertVector2.hpp>

#include <SFML/Graphics/Rect.hpp>


////////////////////////////////////////////////////////////
/// Check if a point is inside a rectangle's area
////////////////////////////////////////////////////////////
bool sfFloatRect_contains(const sfFloatRect* rect, sfVector2f point)
{
    assert(rect);
    return convertRect(*rect).contains(convertVector2(point));
}
bool sfIntRect_contains(const sfIntRect* rect, sfVector2i point)
{
    assert(rect);
    return convertRect(*rect).contains(convertVector2(point));
}


////////////////////////////////////////////////////////////
/// Check intersection between two rectangles
////////////////////////////////////////////////////////////
bool sfFloatRect_intersects(const sfFloatRect* rect1, const sfFloatRect* rect2, sfFloatRect* intersection)
{
    assert(rect1);
    assert(rect2);

    const std::optional overlap = convertRect(*rect1).findIntersection(convertRect(*rect2));

    if (intersection && overlap)
        *intersection = convertRect(*overlap);

    return overlap.has_value();
}
bool sfIntRect_intersects(const sfIntRect* rect1, const sfIntRect* rect2, sfIntRect* intersection)
{
    assert(rect1);
    assert(rect2);

    const std::optional overlap = convertRect(*rect1).findIntersection(convertRect(*rect2));

    if (intersection && overlap)
        *intersection = convertRect(*overlap);

    return overlap.has_value();
}
